#! /usr/local/bin/python
"""
archive.py copys/refreshes/deletes a source directory tree to a target directory.

Pass two arguments please: the full path to the source, and the full path
to the destination: archive.py sourcedir targetdir

Should work on Unix and Windows platforms, it has not been tested on Mac.

Author: John P. Reardon <www.jpreardon.com>
Date:   2004-02-20  
"""

import os, shutil, sys, time
        
def copynew(src, dst):
    """Recursively copy new files in a directory tree using shutil.copy2().
    
    This is based on the shutil module's copytree function, but it
    only copies files if the target is older than the source, or
    if the source does not exist at the target
    
    """
    names = os.listdir(src)
    errors = []
    # If this is a directory, check to see if it exists on the target
    if os.path.isdir(src):
        if False == os.path.exists(dst):
            os.mkdir(dst)
    # Get a list of files and directories to copy
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if os.path.isdir(srcname):
                copynew(srcname, dstname)
            else:
                # Check to see if the target exists
                if os.path.exists(dstname):
                    # Check to see if the target is older than the source
                    if os.path.getmtime(srcname) > os.path.getmtime(dstname):
                        print 'refreshing' ,dstname
                        shutil.copy(srcname, dstname)
                    else:
                        # print dstname, 'is up to date'
                        pass
                else:
                    print 'copying', dstname
                    shutil.copy(srcname, dstname)
                    
        except (IOError, os.error), (errno, errstr):
            errors.append((srcname, dstname, errstr))
    if errors:
        return errors
        
def deletedestorphans(src, dst):
    """Recursively delete files and directories from the destination that are not on the source.
    
    This function is meant to clean up a backup directory, if used improperly
    it could cause serios headaches or data loss. Any file or directory that exists
    on the destination that does not exist on the source will be deleted. Make sure
    the destination is your backup destination!!!
    """
    for root, dirs, files in os.walk(dst, topdown=False):
        for name in files:
            # If non-existant on src, kill
            if not os.path.exists(os.path.join(root, name).replace(dst, src)):
                print 'deleting', os.path.join(root, name)
                os.remove(os.path.join(root, name))
        for name in dirs:
            # If non-existant on src, kill
            if not os.path.exists(os.path.join(root, name).replace(dst,src)):
                print 'deleting', os.path.join(root, name)
                os.rmdir(os.path.join(root, name))

def writelogfile(errs, filepath):
    """Writes a list of errors to a given path.
    
    Pass in the errors as a list and path as a string and this
    baby will write it out to the specified file
    """
    # Open a new file, overwrite if it exists
    file = open(filepath, 'w')
    # Print some information at the top of the file
    file.write('Error log from the archive.py script (' + sys.argv[0].split('\\')[-1] + ')\n')
    file.write('Created: ' + time.strftime('%Y-%m-%d %H:%M:%S') + '\n\n')
    # Write each error on a new line
    for errorline in errs:
        file.write(errorline[0] + ', ' +  errorline[1] + ', ' + errorline[2] + '\n')
    # Close the file
    file.close()

# Main script logic below, run only if script is called from the CLI (thanks Simon!)
if __name__ == '__main__':
    # Get source and target from the commandline
    try:
        sourcepath = sys.argv[1]
    except IndexError:
        print __doc__
        sys.exit()
    try:
        targetpath = sys.argv[2]
    except IndexError:
        print __doc__
        sys.exit()

    # Check to make sure both the source and target paths exist
    if not os.path.exists(sourcepath):
        print sourcepath, 'does not exist!'
        sys.exit()
    if not os.path.exists(targetpath):
        print targetpath, 'does not exist!'
        sys.exit()

    # Copy (or refresh) the source tree on the target 
    print 'Starting backup operation'
    errorlist = copynew(sourcepath, targetpath)

    # Delete files from target that do not exist in the source tree
    print 'Cleaning up'
    deletedestorphans(sourcepath, targetpath)

    # Log errors (if any)
    if errorlist:
        logfilepath = os.path.join(sourcepath,  'archive_err_' + time.strftime('%Y%m%d-%H%M%S') + '.log')
        print 'logging errors to', logfilepath
        writelogfile(errorlist, logfilepath)